# Auto-generated by https://github.com/mos9527/UnityPyTypetreeCodegen
# Python definition for Sekai

from .. import *

@UTTCGen
class PlayerSettingConfig(MonoBehaviour):
	memo : str
	clientMajorVersion : str
	clientMinorVersion : str
	clientBuildVersion : str
	snapshot : str
	clientVersionSuffix : str
	clientDataMajorVersion : str
	clientDataMinorVersion : str
	clientDataBuildVersion : str
	clientDataRevision : str
	companyName : str
	productName : str
	bundleIdentifier : str
	bundleVersion : str
	assetHash : str
	clientAppHash : str
	adMobAppId : str
